#appModules/TTToolBar.py
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2012 NVDA Contributors <http://www.nvda-project.org/>
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.

"""Module for reporting and tracking states of TeamTalk4Classic toolbar checkbox states.

Author:  Doug Lee
"""

import controlTypes, ui, api
from NVDAObjects.IAccessible import IAccessible
import tones
from logHandler import log

# Constants

# The names, in order, of the toolbar items.
# The count must match exactly or this module will not function.
# The name "sep" is a placeholder for separators in the toolbar.
# These are name sets per client version.
_msgNames41 = ["Connected", "sep", "Push To Talk", "Voice Activation", "sep", "Mute All", "sep", "Show Channel Messages"]
_msgNames42 = ["Connected", "sep", "Push To Talk", "Voice Activation", "sep", "Mute All", "Store Audio", "sep", "Show Channel Messages"]
_msgNames43 = ["Connected", "sep", "Push To Talk", "Voice Activation", "Video", "Desktop", "Mute All", "Store Audio", "sep", "Show Channel Messages"]

# How checkboxes on the toolbar are announced when activated (checked) or deactivated.
# %1 is a name from the above list of item names.
_msgActive = "%s active"
_msgInactive = "%s disabled"
_msgConnected = "Connected"
_msgDisconnected = "Disconnected"

class TTToolBarInfo(object):
	"""Info about one TeamTalk toolbar and its items.
	"""
	def __init__(self, NVDAObj=None):
		if not NVDAObj:
			# Use the IAccessible for the active TeamTalk instance's toolbar.
			NVDAObj = self.getToolBar()
			if not NVDAObj:
				ui.message("Toolbar not found")
				return
		self.NVDAObj = NVDAObj
		n = NVDAObj.childCount
		# TeamTalk version-specific toolbar item names by toolbar item count.
		try: self.names = [_msgNames41,_msgNames42,_msgNames43][n-8]
		except IndexError:
			self.names = []

	def getToolBar(self):
		# First lastChild is a Window object, then the client toolbar object.
		tb = api.getForegroundObject().lastChild.lastChild
		return tb

	def name(self, itemID):
		"""The name of one item.
		"""
		if itemID <= 0: return ""
		# ChildIDs are 1-based, but Python arrays are 0-based.
		try: name = self.names[itemID-1]
		except IndexError: return ""
		if name == "sep": return ""
		return name

class TTToolBar(IAccessible):
	def initOverlayClass(self):
		self.info = TTToolBarInfo(self)

class TTToolBarItem(IAccessible):
	def event_valueChange(self):
		speech.speakObject(self)

	@property
	def name(self):
		return self.parent.info.name(self.IAccessibleChildID)

