#appModules/teamtalk4classic.py
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2010-2012 NVDA Contributors <http://www.nvda-project.org/>
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.

import controlTypes
import appModuleHandler
from NVDAObjects.behaviors import ProgressBar
from NVDAObjects.window import edit
import scriptHandler
import ui, speech
import tones
from TTToolBar import TTToolBar, TTToolBarItem, TTToolBarInfo

class AppModule(appModuleHandler.AppModule):
	def event_NVDAObject_init(self,obj):
		# The richedit control displaying incoming chat does not return correct _isWindowUnicode flag.
		if obj.windowClassName=="RichEdit20A":
			obj._isWindowUnicode=False
		# Label main-screen volume controls.
		if obj.windowControlID == 1002:
			obj.name = 'Output Volume'
		elif obj.windowControlID == 1342:
			obj.name = 'Input Volume'
		elif obj.windowControlID == 1001 and obj.role == controlTypes.ROLE_SLIDER:
			obj.name = 'Microphone Activation Level'
		# Label channel chat in/out controls.
		elif obj.windowControlID == 1204:
			obj.name = "Channel messages"
		elif obj.windowControlID == 1206:
			obj.name = "Type a message to the channel"
		# Better name Add/Remove buttons where they occur.
		# Example: Banned Users dialog from the Server menu.
		elif obj.role == controlTypes.ROLE_BUTTON:
			if obj.name == ">":
				obj.name = "Add to unban list"
			elif obj.name == "<":
				obj.name = "Remove from unban list"
		# Name the lists in the Banned Users dialog.
		elif obj.role == controlTypes.ROLE_LIST:
			if obj.windowControlID == 1326:
				obj.name = "Banned"
			elif obj.windowControlID == 1327:
				obj.name = "Unbanned"

	def chooseNVDAObjectOverlayClasses(self, obj, clsList):
		# There is a VU meter progress bar in the main window which we don't want to get anounced as all the other progress bars.
		if obj.windowClassName=="msctls_progress32" and obj.name==u'VU':
			try:
				clsList.remove(ProgressBar)
			except ValueError:
				pass
		# Arrange for incoming chat text to speak.
		if obj.windowControlID == 1204 and obj.windowClassName == 'RichEdit20W':
			clsList.insert(0, ChatText)
		elif obj.windowControlID == 1138 and obj.windowClassName == 'RichEdit20W':
			clsList.insert(0, ReversedChatText)
		# Name toolbar icons.
		elif obj.windowControlID == 59392 and obj.role == controlTypes.ROLE_TOOLBAR:
			clsList.insert(0, TTToolBar)
		elif obj.parent.role == controlTypes.ROLE_TOOLBAR:
			clsList.insert(0, TTToolBarItem)

	def script_sayToolbarInfo(self, gesture):
		"""Say various on/off states from the toolbar. Depending
		on TeamTalk version, these can include connection status,
		status of voice activation, push-to-talk, and video
		features, whether channel audio is being saved to files,
		and whether the channel message window is showing.
		On one press, just says which are checked (in effect).
		On two presses, says all states.
		"""
		info = TTToolBarInfo()
		if not info: return
		scnt = scriptHandler.getLastScriptRepeatCount()
		if scnt > 0:
			[speech.speakObject(o)
				for o in info.NVDAObj.children
				if o.role == controlTypes.ROLE_CHECKBOX
				and not (controlTypes.STATE_UNAVAILABLE in o.states)
			]
			return
		for o in info.NVDAObj.children:
			if o.role != controlTypes.ROLE_CHECKBOX: continue
			if (controlTypes.STATE_UNAVAILABLE in o.states): continue
			if controlTypes.STATE_CHECKED not in o.states: continue
			speech.speakText(o.name)

	__gestures = {
		"kb:control+shift+s": "sayToolbarInfo"
	}

class ChatText(edit.Edit):
	reversed = False
	lastLengths = {}
	def event_valueChange(self):
		text=self.windowText
		wlen = len(text)
		hwnd = self.windowHandle
		# ToDo: This would cause new chat windows to speak when created
		# if a ValueChange event fired then,
		# but it also causes new messages after an NVDA restart to
		# speak every line sent to that window before NVDA was
		# restarted. The next line would fix that but make the first
		# line sent to any window silent.
		self.lastLengths.setdefault(hwnd, 0)
		#self.lastLengths.setdefault(hwnd, wlen)
		if wlen == self.lastLengths[hwnd]: return
		lastLength = self.lastLengths[hwnd]
		self.lastLengths[hwnd] = wlen
		if self.reversed:
			new_text = text[:-lastLength-1]
		else:
			new_text = text[lastLength:]
		if new_text == "": #no new text was added this time
			return
		ui.message(new_text)

class ReversedChatText(ChatText):
	reversed = True
